﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports System
Imports Microsoft.VisualBasic
Imports System.Linq.Expressions
Imports System.Reflection
Imports System.Web.Mvc
Imports System.Web.Routing

Namespace Helper

    Friend NotInheritable Class LinkBuilder
        Private Sub New()
        End Sub
        Public Shared Function BuildUrlFromExpression(Of TController As Controller)(ByVal context As RequestContext, ByVal routeCollection As RouteCollection, ByVal action As Expression(Of Action(Of TController))) As String
            Dim routeValuesFromExpression As RouteValueDictionary = GetRouteValuesFromExpression(Of TController)(action)
            Dim virtualPath As VirtualPathData = routeCollection.GetVirtualPath(context, routeValuesFromExpression)
            If virtualPath IsNot Nothing Then
                Return virtualPath.VirtualPath
            End If

            Return Nothing
        End Function

        Private Shared Function GetRouteValuesFromExpression(Of TController As Controller)(ByVal action As Expression(Of Action(Of TController))) As RouteValueDictionary
            If action Is Nothing Then
                Throw New ArgumentNullException("action")
            End If

            Dim body As MethodCallExpression = TryCast(action.Body, MethodCallExpression)
            If body Is Nothing Then
                Throw New ArgumentException("Expression must be a method call.", "action")
            End If

            Dim name As String = GetType(TController).Name
            If (Not name.EndsWith("Controller", StringComparison.OrdinalIgnoreCase)) Then
                Throw New ArgumentException("Controller name must end in 'Controller'.", "action")
            End If

            name = name.Substring(0, name.Length - "Controller".Length)
            If name.Length = 0 Then
                Throw New ArgumentException("Cannot route to class named 'Controller'.", "action")
            End If

            Dim rvd As New RouteValueDictionary()
            rvd.Add("Controller", name)
            rvd.Add("Action", body.Method.Name)
            AddParameterValuesFromExpressionToDictionary(rvd, body)
            Return rvd
        End Function

        Private Shared Sub AddParameterValuesFromExpressionToDictionary(ByVal rvd As RouteValueDictionary, ByVal [call] As MethodCallExpression)
            Dim parameters() As ParameterInfo = [call].Method.GetParameters()
            If parameters.Length > 0 Then
                For i As Integer = 0 To parameters.Length - 1
                    Dim expression As Expression = [call].Arguments(i)
                    Dim obj2 As Object = Nothing
                    Dim expression2 As ConstantExpression = TryCast(expression, ConstantExpression)
                    If expression2 IsNot Nothing Then
                        obj2 = expression2.Value
                    Else
                        Dim expression3 As Expression(Of Func(Of Object)) = expression.Lambda(Of Func(Of Object))(expression.Convert(expression, GetType(Object)), New ParameterExpression() {})
                        obj2 = expression3.Compile()()
                    End If

                    rvd.Add(parameters(i).Name, obj2)
                Next i
            End If
        End Sub
    End Class
End Namespace
