﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports Microsoft.VisualBasic

Imports System.Collections.Generic
Imports System
Imports System.IO
Imports System.Linq
Imports System.Net
Imports System.Xml.Linq
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Microsoft.WindowsAzure.StorageClient
Imports MyTodo.Data.WindowsAzure

Namespace Models

    Public Class TaskRepository
        Public Overridable Function TablesExist() As Boolean
            Dim context = New TaskDataContext()
            Return context.IsTablesCreated
        End Function

        Public Overridable Sub CreateTables()
            Dim context = New TaskDataContext()
            context.CreateTables()
        End Sub

        Public Overridable Function CreateList(ByVal name As String, ByVal isPublic As Boolean) As TaskList
            Dim context = New TaskDataContext()
            Dim list = New TaskList With {.Name = name, .IsPublic = isPublic}
            context.AddObject(TaskDataContext.TaskListTable, list.ToRow())
            context.SaveChanges()

            Return list
        End Function

        Public Overridable Sub CreateTask(ByVal task As Task)
            Dim context = New TaskDataContext()

            context.AddObject(TaskDataContext.TaskTable, task.ToRow())
            context.SaveChanges()
        End Sub

        Public Overridable Sub DeleteTask(ByVal listId As String, ByVal taskId As String)
            Dim context = New TaskDataContext()

            Dim task = context.Tasks.Where(Function(t) t.PartitionKey = listId AndAlso t.RowKey = taskId AndAlso True).SingleOrDefault()

            If task IsNot Nothing Then
                context.DeleteObject(task)
                context.SaveChanges()
            End If
        End Sub

        Public Overridable Sub UpdateTask(ByVal task As Task)
            Dim context = New TaskDataContext()
            Dim taskRow = task.ToRow()

            context.AttachTo(TaskDataContext.TaskTable, taskRow, "*")
            context.UpdateObject(taskRow)
            context.SaveChanges()
        End Sub

        Public Overridable Function GetTask(ByVal taskId As String, ByVal listId As String) As Task
            Dim context = New TaskDataContext()

            Dim task = context.Tasks.Where(Function(t) t.RowKey = taskId AndAlso t.PartitionKey = listId AndAlso True).ToModel().SingleOrDefault()

            Return task
        End Function

        Public Overridable Sub UpdateList(ByVal list As TaskList)
            Dim context = New TaskDataContext()
            Dim taskListRow = list.ToRow()

            context.AttachTo(TaskDataContext.TaskListTable, taskListRow, "*")
            context.UpdateObject(taskListRow)
            context.SaveChanges()
        End Sub

        Public Overridable Function GetTaskLists(ByVal includePrivate As Boolean) As IEnumerable(Of TaskList)
            Dim context = New TaskDataContext()

            Dim lists = context.TaskLists.Where(Function(t) t.IsPublic = True OrElse t.IsPublic <> includePrivate)

            Return lists.ToModel()
        End Function

        Public Overridable Function GetTaskList(ByVal listId As String) As TaskList
            Dim context = New TaskDataContext()

            If Me.TablesExist() Then
                Dim list = context.TaskLists.Where(Function(t) t.RowKey = listId AndAlso t.PartitionKey = listId.Substring(0, 1) AndAlso True).ToModel().SingleOrDefault()

                Return list
            End If

            Return Nothing
        End Function

        Public Overridable Sub DeleteList(ByVal listId As String)
            Dim context = New TaskDataContext()

            Dim list = context.TaskLists.Where(Function(t) t.RowKey = listId AndAlso t.PartitionKey = listId.Substring(0, 1) AndAlso True).SingleOrDefault()

            If list IsNot Nothing Then
                Dim tasks = context.Tasks.Where(Function(t) t.PartitionKey = listId)

                For Each task As TaskRow In tasks
                    context.DeleteObject(task)
                Next task

                context.DeleteObject(list)
                context.SaveChanges()
            End If
        End Sub

        Public Function GetTasks(ByVal listId As String, ByVal pageSize As Integer) As PagedTaskSource
            Return Me.GetTasks(listId, pageSize, Nothing, Nothing)
        End Function

        Public Overridable Function GetTasks(ByVal listId As String, ByVal pageSize As Integer, ByVal nextPartition As String, ByVal nextRowKey As String) As PagedTaskSource
            Dim context = New TaskDataContext()
            Return New PagedTaskSource(context.Tasks.Where(Function(t) t.ListId = listId), nextPartition, nextRowKey, pageSize)
        End Function

        Public Overridable Function RetrieveInitialLists() As XDocument
            Dim client = New WebClient()
            Dim blobUrl = RoleEnvironment.GetConfigurationSettingValue("InitialListsBlobUrl")

            Try
                Using data = client.OpenRead(blobUrl)
                    Using reader = New StreamReader(data)
                        Dim blobContent = reader.ReadToEnd()

                        Return XDocument.Parse(blobContent)
                    End Using
                End Using
            Catch
            End Try

            Return New XDocument()
        End Function

        Public Sub CreatePublicBlobForInitialData(ByVal containerName As String, ByVal blobName As String)
            Dim blobAccount As CloudStorageAccount = CloudStorageAccount.FromConfigurationSetting("DataConnectionString")

            Dim blobStorage As CloudBlobClient = blobAccount.CreateCloudBlobClient()
            blobStorage.RetryPolicy = RetryPolicies.Retry(1, TimeSpan.FromMilliseconds(100))

            Dim container As CloudBlobContainer = blobStorage.GetContainerReference(containerName.ToLowerInvariant())
            container.CreateIfNotExist()
            Dim permissions = container.GetPermissions()
            permissions.PublicAccess = BlobContainerPublicAccessType.Container
            container.SetPermissions(permissions)

            Dim blob As CloudBlockBlob = container.GetBlockBlobReference(blobName)
            blob.UploadByteArray(New Byte() {})
        End Sub
    End Class
End Namespace