﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------


Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.StorageClient
Imports MyTodo.Data.WindowsAzure.Microsoft.WindowsAzure.StorageClient


Public Class TaskDataContext
    Inherits TableServiceContext
    Public Const TaskTable = "Tasks"
    Public Const TaskListTable = "TaskLists"
    Private ReadOnly resolverTypes As Dictionary(Of String, Type)

    Public Sub New()
        Me.New(CloudStorageAccount.FromConfigurationSetting("DataConnectionString").TableEndpoint.AbsoluteUri, CloudStorageAccount.FromConfigurationSetting("DataConnectionString").Credentials)
    End Sub

    Public Sub New(ByVal baseAddress As String, ByVal credentials As StorageCredentials)
        MyBase.New(baseAddress, credentials)
        ' we are setting up a dictionary of types to resolve in order
        ' to workaround a performance bug in astoria during serialization
        Me.resolverTypes = New Dictionary(Of String, Type)()
        Me.resolverTypes.Add(TaskTable, GetType(TaskRow))
        Me.resolverTypes.Add(TaskListTable, GetType(TaskListRow))

        Me.ResolveType = Function(name) AnonymousMethod1(name)

    End Sub

    Private Function AnonymousMethod1(ByVal name As String) As Type
        Dim parts = name.Split("."c)
        If parts.Length = 2 Then
            Return resolverTypes(parts(1))
        End If
        Return Nothing
    End Function

    Public ReadOnly Property IsTablesCreated() As Boolean
        Get
            Dim client = New CloudTableClient(Me.BaseUri.AbsoluteUri, Me.StorageCredentials)
            Return client.DoesTableExist(TaskTable) AndAlso client.DoesTableExist(TaskListTable)
        End Get
    End Property

    Public ReadOnly Property Tasks() As IQueryable(Of TaskRow)
        Get
            Return Me.CreateQuery(Of TaskRow)(TaskTable)
        End Get
    End Property

    Public ReadOnly Property TaskLists() As IQueryable(Of TaskListRow)
        Get
            Return Me.CreateQuery(Of TaskListRow)(TaskListTable)
        End Get
    End Property

    Public Sub CreateTables()
        TableStorageExtensionMethods.CreateTablesFromModel(GetType(TaskDataContext), Me.BaseUri.AbsoluteUri, Me.StorageCredentials)
    End Sub
End Class
