﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------


Imports Microsoft.VisualBasic

Imports System.Web
Imports System.Web.Mvc

Namespace Controllers

    Public Class CacheFilterAttribute
        Inherits ActionFilterAttribute
        Public Sub New()
            Me.Duration = 0
        End Sub

        Private privateDuration As Integer
        Public Property Duration() As Integer
            Get
                Return privateDuration
            End Get
            Set(ByVal value As Integer)
                privateDuration = value
            End Set
        End Property

        Public Overrides Sub OnActionExecuted(ByVal filterContext As ActionExecutedContext)
            If Me.Duration = 0 Then
                Return
            End If

            Dim cache As HttpCachePolicyBase = filterContext.HttpContext.Response.Cache

            If Me.Duration < 0 Then
                cache.SetCacheability(HttpCacheability.NoCache)
                cache.SetExpires(DateTime.Now.AddDays(-1))
            Else
                Dim cacheDuration As TimeSpan = TimeSpan.FromSeconds(Me.Duration)
                cache.SetCacheability(HttpCacheability.Public)
                cache.SetExpires(DateTime.Now.Add(cacheDuration))
                cache.SetMaxAge(cacheDuration)
                cache.AppendCacheExtension("must-revalidate, proxy-revalidate")
            End If
        End Sub
    End Class
End Namespace