﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------


Imports Microsoft.VisualBasic
Imports System.Collections.Generic
Imports System.Data.Services.Client
Imports System.Linq
Imports MyTodo.Data.WindowsAzure

Namespace Models

    Public Class PagedTaskSource
        Public Sub New(ByVal source As IQueryable(Of TaskRow), ByVal partition As String, ByVal rowKey As String, ByVal pageSize As Integer)
            Me.PageSize = pageSize

            Dim query = TryCast(source.Take(Me.PageSize), DataServiceQuery(Of TaskRow))

            If query IsNot Nothing Then
                If partition IsNot Nothing AndAlso rowKey IsNot Nothing Then
                    query = query.AddQueryOption("NextPartitionKey", partition).AddQueryOption("NextRowKey", rowKey)
                End If

                ' set the Tasks
                Dim res = query.Execute()
                Me.Tasks = res.ToModel()

                ' get the continuation markers
                Dim nextPartition As String
                Dim nextRow As String

                Dim headers = (CType(res, QueryOperationResponse)).Headers

                headers.TryGetValue("x-ms-continuation-NextPartitionKey", nextPartition)
                headers.TryGetValue("x-ms-continuation-NextRowKey", nextRow)

                ' and return them to the client
                Me.NextPartition = nextPartition
                Me.NextRow = nextRow
            End If
        End Sub

        Protected Sub New()
        End Sub

        Private privatePageSize As Integer
        Public Property PageSize() As Integer
            Get
                Return privatePageSize
            End Get
            Set(ByVal value As Integer)
                privatePageSize = value
            End Set
        End Property

        Private privateNextPartition As String
        Public Property NextPartition() As String
            Get
                Return privateNextPartition
            End Get
            Private Set(ByVal value As String)
                privateNextPartition = value
            End Set
        End Property

        Private privateNextRow As String
        Public Property NextRow() As String
            Get
                Return privateNextRow
            End Get
            Private Set(ByVal value As String)
                privateNextRow = value
            End Set
        End Property

        Private privateTasks As IEnumerable(Of Task)
        Public Overridable Property Tasks() As IEnumerable(Of Task)
            Get
                Return privateTasks
            End Get
            Set(ByVal value As IEnumerable(Of Task))
                privateTasks = value
            End Set
        End Property
    End Class
End Namespace