﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------


Imports Microsoft.VisualBasic
Imports System.Globalization
Imports System.Text
Imports System.Web


Public NotInheritable Class SlugHelper
    Private Sub New()
    End Sub
    <System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Globalization", "CA1308:NormalizeStringsToUppercase", Justification:="Slug are always lowercase.")> _
    Public Shared Function GetSlug(ByVal text As String) As String
        If String.IsNullOrEmpty(text) Then
            Return text
        End If

        text = text.ToLowerInvariant()
        text = text.Replace(":", String.Empty)
        text = text.Replace("/", String.Empty)
        text = text.Replace("?", String.Empty)
        text = text.Replace("¿", String.Empty)
        text = text.Replace("!", String.Empty)
        text = text.Replace("¡", String.Empty)
        text = text.Replace("#", String.Empty)
        text = text.Replace("[", String.Empty)
        text = text.Replace("]", String.Empty)
        text = text.Replace("@", String.Empty)
        text = text.Replace("*", String.Empty)
        text = text.Replace(".", String.Empty)
        text = text.Replace(",", String.Empty)
        text = text.Replace("""", String.Empty)
        text = text.Replace("&", String.Empty)
        text = text.Replace("'", String.Empty)
        text = text.Replace(" ", "-")
        text = RemoveDiacritics(text)
        text = RemoveExtraHyphen(text)

        Return HttpUtility.UrlEncode(text).Replace("%", String.Empty)
    End Function

    Private Shared Function RemoveExtraHyphen(ByVal text As String) As String
        If text.Contains("--") Then
            text = text.Replace("--", "-")
            Return RemoveExtraHyphen(text)
        End If

        Return text
    End Function

    Private Shared Function RemoveDiacritics(ByVal text As String) As String
        Dim normalized As String = text.Normalize(NormalizationForm.FormD)
        Dim sb As New StringBuilder()

        For i As Integer = 0 To normalized.Length - 1
            Dim c As Char = normalized.Chars(i)
            If CharUnicodeInfo.GetUnicodeCategory(c) <> UnicodeCategory.NonSpacingMark Then
                sb.Append(c)
            End If
        Next i

        Return sb.ToString()
    End Function
End Class
