IF OBJECT_ID('[dbo].[SetForeignKeyEnabledStatus]','P') IS NOT NULL 
    DROP PROCEDURE [dbo].[SetForeignKeyEnabledStatus]; 
GO
CREATE PROCEDURE [dbo].[SetForeignKeyEnabledStatus] 
( 
    @enabled bit 
) 
AS 
BEGIN 
    DECLARE 
        @schema_name sysname, 
        @table_name sysname, 
        @fk_constraint_name sysname, 
        @cmd_txt varchar(8000);
DECLARE fk_cursor CURSOR FOR 
        SELECT 
            sc.name AS schema_name, 
            so.name AS table_name, 
            so2.name AS fk_constraint_name 
        FROM 
            sys.objects so 
            inner join sys.schemas sc ON so.schema_id = sc.schema_id 
            inner join sys.foreign_key_columns fk ON so.object_id = fk.parent_object_id 
            inner join sys.objects so2 ON so2.object_id = fk.constraint_object_id 
        WHERE 
            so.type = 'U';
OPEN fk_cursor;
FETCH NEXT FROM fk_cursor INTO 
        @schema_name, 
        @table_name, 
        @fk_constraint_name;
WHILE @@FETCH_STATUS = 0 
    BEGIN 
        IF @enabled = 0 
            SET @cmd_txt = 'ALTER TABLE [' + @schema_name + '].[' + @table_name + '] NOCHECK CONSTRAINT [' + @fk_constraint_name + '];' 
        ELSE 
            SET @cmd_txt = 'ALTER TABLE [' + @schema_name + '].[' + @table_name + '] WITH CHECK CHECK CONSTRAINT [' + @fk_constraint_name + '];';                     
        PRINT @cmd_txt; 
        EXECUTE (@cmd_txt); 
        FETCH NEXT FROM fk_cursor INTO 
            @schema_name, 
            @table_name, 
            @fk_constraint_name; 
    END;
CLOSE fk_cursor; 
    DEALLOCATE fk_cursor; 
END; 
GO
IF OBJECT_ID('[dbo].[SetIndexEnabledStatus]','P') IS NOT NULL 
    DROP PROCEDURE [dbo].[SetIndexEnabledStatus]; 
GO
CREATE PROCEDURE [dbo].[SetIndexEnabledStatus] 
( 
    @enabled bit 
) 
AS 
BEGIN 
    DECLARE 
        @schema_name sysname, 
        @table_name sysname, 
        @index_name sysname, 
        @cmd_txt varchar(8000);
DECLARE idx_cursor CURSOR FOR 
        SELECT 
            sc.name AS schema_name, 
            so.name AS table_name, 
            si.name AS index_name 
        FROM 
            sys.objects so 
            inner join sys.schemas sc ON so.schema_id = sc.schema_id 
            inner join sys.indexes si ON so.object_id = si.object_id 
        WHERE 
            so.type = 'U' 
            AND si.index_id > 1; 
    OPEN idx_cursor;
FETCH NEXT FROM idx_cursor INTO 
        @schema_name, 
        @table_name, 
        @index_name;
WHILE @@FETCH_STATUS = 0 
    BEGIN 
        IF @enabled = 0        
            SET @cmd_txt = 'ALTER INDEX [' + @index_name + '] ON [' + @schema_name + '].[' + @table_name + '] DISABLE;' 
        ELSE 
            SET @cmd_txt = 'ALTER INDEX [' + @index_name + '] ON [' + @schema_name + '].[' + @table_name + '] REBUILD;' 
        PRINT @cmd_txt; 
        EXECUTE (@cmd_txt); 
        FETCH NEXT FROM idx_cursor INTO 
            @schema_name, 
            @table_name, 
            @index_name; 
    END;
CLOSE idx_cursor; 
    DEALLOCATE idx_cursor; 
END; 
GO