﻿' ----------------------------------------------------------------------------------
' Microsoft Developer & Platform Evangelism
' 
' Copyright (c) Microsoft Corporation. All rights reserved.
' 
' THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
' EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
' OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
' ----------------------------------------------------------------------------------
' The example companies, organizations, products, domain names,
' e-mail addresses, logos, people, places, and events depicted
' herein are fictitious.  No association with any real company,
' organization, product, domain name, email address, logo, person,
' places, or events is intended or should be inferred.
' ----------------------------------------------------------------------------------

Imports Microsoft.WindowsAzure
Imports Microsoft.WindowsAzure.ServiceRuntime
Imports Microsoft.WindowsAzure.StorageClient

Partial Public Class _Default
    Inherits System.Web.UI.Page

    Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs) Handles Me.Load
        Try
            If Not IsPostBack Then
                Me.EnsureContainerExists()
            End If
            Me.RefreshGallery()
        Catch we As System.Net.WebException
            status.Text = "Network error: " & we.Message
            If we.Status = System.Net.WebExceptionStatus.ConnectFailure Then
                status.Text += "<br />Please check if the blob service is running at " & ConfigurationManager.AppSettings("storageEndpoint")
            End If
        Catch se As StorageException
            Console.WriteLine("Storage service error: " & se.Message)
        End Try
    End Sub

    Protected Sub upload_Click(ByVal sender As Object, ByVal e As EventArgs)
        If imageFile.HasFile Then
            status.Text = (("Inserted [" & imageFile.FileName & "] - Content Type [") + imageFile.PostedFile.ContentType & "] - Length [") + imageFile.PostedFile.ContentLength.ToString() + "]"

            Me.SaveImage(Guid.NewGuid().ToString(), imageName.Text, imageDescription.Text, imageTags.Text, imageFile.FileName, imageFile.PostedFile.ContentType, _
              imageFile.FileBytes)

            RefreshGallery()
        Else
            status.Text = "No image file"
        End If
    End Sub


    Protected Sub OnBlobDataBound(ByVal sender As Object, ByVal e As ListViewItemEventArgs) Handles images.ItemDataBound
        If e.Item.ItemType = ListViewItemType.DataItem Then
            Dim metadataRepeater = TryCast(e.Item.FindControl("blobMetadata"), Repeater)
            Dim blob = TryCast(DirectCast((e.Item), ListViewDataItem).DataItem, CloudBlob)

            ' If this blob is a snapshot, rename button to "Delete Snapshot"
            If blob IsNot Nothing Then
                If blob.SnapshotTime.HasValue Then
                    Dim delBtn = TryCast(e.Item.FindControl("deleteBlob"), LinkButton)
                    If delBtn IsNot Nothing Then
                        delBtn.Text = "Delete Snapshot"
                    End If

                    Dim snapshotBtn = TryCast(e.Item.FindControl("Snapshot"), LinkButton)
                    If snapshotBtn IsNot Nothing Then
                        snapshotBtn.Visible = False
                    End If
                End If

                If metadataRepeater IsNot Nothing Then
                    'bind to metadata
                    metadataRepeater.DataSource = From key In blob.Metadata.AllKeys _
                    Select New With {.Name = key, .Value = blob.Metadata.Get(key)}
                    metadataRepeater.DataBind()
                End If
            End If
        End If
    End Sub


    Protected Sub OnDeleteImage(ByVal sender As Object, ByVal e As CommandEventArgs)
        Try
            If e.CommandName = "Delete" Then
                Dim blobUri = DirectCast(e.CommandArgument, String)
                Dim blob = Me.GetContainer().GetBlobReference(blobUri)
                blob.DeleteIfExists()
            End If
        Catch se As StorageClientException
            status.Text = "Storage client error: " & se.Message
        Catch generatedExceptionName As Exception
        End Try

        RefreshGallery()
    End Sub


    Protected Sub OnCopyImage(ByVal sender As Object, ByVal e As CommandEventArgs)
        If e.CommandName = "Copy" Then
            ' Prepare an Id for the copied blob
            Dim newId = Guid.NewGuid()

            ' Get source blob
            Dim blobUri = DirectCast(e.CommandArgument, String)
            Dim srcBlob = Me.GetContainer().GetBlobReference(blobUri)

            ' Create new blob
            Dim newBlob = Me.GetContainer().GetBlobReference(newId.ToString())

            ' Copy content from source blob
            newBlob.CopyFromBlob(srcBlob)

            ' Explicitly get metadata for new blob
            newBlob.FetchAttributes(New BlobRequestOptions())

            ' Change metadata on the new blob to reflect this is a copy via UI
            newBlob.Metadata("ImageName") = "Copy of """ & newBlob.Metadata("ImageName") & """"
            newBlob.Metadata("Id") = newId.ToString()
            newBlob.SetMetadata()

            ' Render all blobs
            RefreshGallery()
        End If
    End Sub

    Protected Sub OnSnapshotImage(ByVal sender As Object, ByVal e As CommandEventArgs)
        If e.CommandName = "Snapshot" Then
            ' Get source blob
            Dim blobUri = DirectCast(e.CommandArgument, String)
            Dim srcBlob = Me.GetContainer().GetBlobReference(blobUri)

            ' Create a snapshot
            Dim snapshot = srcBlob.CreateSnapshot()

            status.Text = "A snapshot has been taken for image blob:" + srcBlob.Uri.ToString() + " at " + snapshot.SnapshotTime

            RefreshGallery()
        End If
    End Sub

    Private Sub EnsureContainerExists()
        Dim container = GetContainer()
        container.CreateIfNotExist()

        Dim permissions = container.GetPermissions()
        permissions.PublicAccess = BlobContainerPublicAccessType.Container
        container.SetPermissions(permissions)
    End Sub

    Private Function GetContainer() As CloudBlobContainer
        ' Get a handle on account, create a blob service client and get container proxy
        Dim account = CloudStorageAccount.FromConfigurationSetting("DataConnectionString")
        Dim client = account.CreateCloudBlobClient()

        Return client.GetContainerReference(RoleEnvironment.GetConfigurationSettingValue("ContainerName"))
    End Function

    Private Sub RefreshGallery()
        Dim options As BlobRequestOptions = New BlobRequestOptions() With
          {.BlobListingDetails = BlobListingDetails.All,
           .UseFlatBlobListing = True}

        images.DataSource = Me.GetContainer().ListBlobs(options)
        images.DataBind()
    End Sub

    Private Sub SaveImage(ByVal id As String, ByVal name As String, ByVal description As String, ByVal tags As String, ByVal fileName As String, ByVal contentType As String, _
      ByVal data As Byte())
        ' Create a blob in container and upload image bytes to it
        Dim blob = Me.GetContainer().GetBlobReference(name)

        blob.Properties.ContentType = contentType

        ' Create some metadata for this image
        Dim metadata = New NameValueCollection()
        metadata("Id") = id
        metadata("Filename") = fileName
        metadata("ImageName") = If([String].IsNullOrEmpty(name), "unknown", name)
        metadata("Description") = If([String].IsNullOrEmpty(description), "unknown", description)
        metadata("Tags") = If([String].IsNullOrEmpty(tags), "unknown", tags)

        ' Add and commit metadata to blob
        blob.Metadata.Add(metadata)
        blob.UploadByteArray(data)
    End Sub
End Class

